﻿using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Linq;
using System.Windows;

namespace PropertyGridTypeConverter
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            this.propertyGrid.SelectedObject = new PropertyOwner(new List<string> {"Item 1", "Item2", "Item3"});
        }
    }

    /// <summary>
    /// Sample data model object to view in property editor.
    /// </summary>
    public class PropertyOwner
    {
        private string option;

        public PropertyOwner(List<string> dynamicOptions)
        {
            this.option = dynamicOptions.First();
            this.Options = dynamicOptions;
        }

        [DisplayName("My Option")]
        [Category("My Category")]
        [TypeConverter(typeof(MyValueTypeConverter))]
        public string Option
        {
            get { return this.option; }

            set
            {
                this.option = value;
                Debug.WriteLine($"Option set to {this.option}");
            }
            
        }

        internal List<string> Options { get; }
    }

    public class MyValueTypeConverter : TypeConverter
    {
        public override TypeConverter.StandardValuesCollection GetStandardValues(ITypeDescriptorContext context)
        {
#if USE_CONTEXT
            // Build list of options for the combo-box editor dynamically from model data through the context.
            Debug.Assert(context != null, "Invalid context");
            string[] myValues = (context.Instance as PropertyOwner).Options.ToArray();
#else
            // Need to hardcode values, or use some dirty trick using e.g. static/global variables.
            string[] myValues = { "Item 1", "Item 2", "Item 3" }; 
#endif
            return new StandardValuesCollection(myValues);
        }

        public override bool GetStandardValuesSupported(ITypeDescriptorContext context)
        {
            return true;
        }

        public override bool GetStandardValuesExclusive(ITypeDescriptorContext context)
        {
            return true;
        }
    }
}
